use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
drolsky
DROLSKY
DROLSKY's
PayPal
Rolsky
Rolsky's
ini
mailmap
optimizations
perlcriticrc
perltidyrc
tidyall
Dave
autarch
lib
Dist
Zilla
Plugin
CheckChangesHasContent
Contributors
Git
CheckFor
CorrectBranch
License
TidyAll
VersionProvider
PluginBundle
Pod
Weaver
